@@ -46,7 +46,7 @@ def inv(a, n):
 # Base switching
 code_strings = {
     2: '01',
-    10: '012356789',
+    10: '0123456789',
     16: '0123456789abcdef',
     32: 'abcdefghijklmnopqrstuvwxyz234567',
     58: '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz',
@@ -406,6 +406,18 @@ def sha256(string):
     return binascii.hexlify(bin_sha256(string))
 
 
+def bin_ripemd160(string):
+    try:
+        digest = hashlib.new('ripemd160', string).digest()
+    except:
+        digest = ripemd.RIPEMD160(string).digest()
+    return digest
+
+
+def ripemd160(string):
+    return binascii.hexlify(bin_ripemd160(string))
+
+
 def bin_dbl_sha256(string):
     return hashlib.sha256(hashlib.sha256(string).digest()).digest()
 
@@ -545,6 +557,8 @@ def ecdsa_sign(msg, priv):
 
 def ecdsa_raw_verify(msghash, vrs, pub):
     v, r, s = vrs
+    if v < 27 or v > 28:
+        return False
 
     w = inv(s, N)
     z = hash_to_int(msghash)
@@ -561,6 +575,8 @@ def ecdsa_verify(msg, sig, pub):
 
 def ecdsa_raw_recover(msghash, vrs):
     v, r, s = vrs
+    if v < 27 or v > 28:
+        return False
 
     x = r
     beta = pow(x*x*x+A*x+B, (P+1)/4, P)
